# Copyright (c) 2017 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from openstack_dashboard.test import helpers

from bgpvpn_dashboard.api import bgpvpn as bgpvpn_api
from bgpvpn_dashboard.dashboards.admin.bgpvpn import forms as bgpvpn_a_form
from bgpvpn_dashboard.dashboards.project.bgpvpn import forms as bgpvpn_p_form


class Tenant(object):
    def __init__(self, id, name, enabled):
        self.id = id
        self.name = name
        self.enabled = enabled


class TestCreateDataBgpVpn(helpers.APITestCase):

    @mock.patch.object(bgpvpn_a_form, 'api')
    def setUp(self, mock_api):
        super(TestCreateDataBgpVpn, self).setUp()
        self.mock_request = mock.MagicMock()

    @mock.patch.object(bgpvpn_p_form, 'bgpvpn_api')
    @mock.patch.object(bgpvpn_a_form, 'api')
    def test_invalid_rt(self, mock_api, mock_bgpvpn_api):
        mock_api.keystone.tenant_list.return_value = [], False
        mock_bgpvpn_api.list_bgpvpns.return_value = []
        data = {"route_targets": "xyz",
                "import_targets": "0",
                "export_targets": "64512:1000000000000, xyz"}
        self.bgpvpn_form = bgpvpn_a_form.CreateBgpVpn(self.mock_request, data)

        self.assertTrue(self.bgpvpn_form.has_error("route_targets"))
        self.assertTrue(self.bgpvpn_form.has_error("import_targets"))
        self.assertTrue(self.bgpvpn_form.has_error("export_targets"))

    @mock.patch.object(bgpvpn_p_form, 'bgpvpn_api')
    @mock.patch.object(bgpvpn_a_form, 'api')
    def test_valid_rt(self, mock_api, mock_bgpvpn_api):
        mock_api.keystone.tenant_list.return_value = [], False
        mock_bgpvpn_api.list_bgpvpns.return_value = []
        data = {"route_targets": "65421:1",
                "import_targets": "65421:1, 65421:2",
                "export_targets": "65421:3"}
        self.bgpvpn_form = bgpvpn_a_form.CreateBgpVpn(self.mock_request, data)

        self.assertFalse(self.bgpvpn_form.has_error("route_targets"))
        self.assertFalse(self.bgpvpn_form.has_error("import_targets"))
        self.assertFalse(self.bgpvpn_form.has_error("export_targets"))

    @mock.patch.object(bgpvpn_p_form, 'bgpvpn_api')
    @mock.patch.object(bgpvpn_a_form, 'api')
    def test_handle_update(self, mock_api, mock_bgpvpn_api):
        data = {"bgpvpn_id": "foo-id",
                "type": "l3",
                "name": "foo-name",
                "route_targets": "65421:1",
                "import_targets": "65421:2",
                "export_targets": "65421:3"}
        mock_api.keystone.tenant_list.return_value = [], False
        self.bgpvpn_form = bgpvpn_a_form.CreateBgpVpn(self.mock_request)
        self.bgpvpn_form.action = "update"
        expected_data = bgpvpn_api.Bgpvpn({"bgpvpn_id": "foo-id",
                                           "name": "foo-name",
                                           "tenant_id": "tenant_id",
                                           "route_targets": ["65421:1"],
                                           "import_targets": ["65421:2"],
                                           "export_targets": ["65421:3"]})
        mock_bgpvpn_api.bgpvpn_update.return_value = expected_data

        result = self.bgpvpn_form.handle(self.mock_request, data)

        mock_bgpvpn_api.bgpvpn_update.assert_called_once_with(
            self.mock_request, "foo-id",
            name="foo-name",
            route_targets=["65421:1"],
            import_targets=["65421:2"],
            export_targets=["65421:3"])
        self.assertEqual(result, expected_data)



# Copyright 2017 <PUT YOUR NAME/COMPANY HERE>
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from alembic import op
import sqlalchemy as sa

"""Add 'extra-routes' to router association table

Revision ID: 4610803bdf0d
Revises: 9a6664f3b8d4
Create Date: 2017-06-26 17:39:11.086696

"""

# revision identifiers, used by Alembic.
revision = '4610803bdf0d'
down_revision = '39411aacf9b8'


def upgrade():
    op.add_column('bgpvpn_router_associations',
                  sa.Column('advertise_extra_routes',
                            sa.Boolean(), nullable=False,
                            server_default=sa.true()))



# Copyright 2018 <PUT YOUR NAME/COMPANY HERE>
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from alembic import op
import sqlalchemy as sa

""" Add local_pref to bgpvpns table

Revision ID: 666c706fea3b
Revises: 39411aacf9b8
Create Date: 2018-01-18 15:40:05.723129

"""

# revision identifiers, used by Alembic.
revision = '666c706fea3b'
down_revision = '4610803bdf0d'


def upgrade():
    op.add_column('bgpvpns', sa.Column('local_pref',
                                       sa.BigInteger,
                                       nullable=True))



# Copyright (c) 2015 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_db import exception as db_exc
from oslo_log import log
from oslo_utils import uuidutils
import sqlalchemy as sa
from sqlalchemy import orm
from sqlalchemy.orm import exc

from neutron_lib.api.definitions import bgpvpn as bgpvpn_def
from neutron_lib.api.definitions import bgpvpn_routes_control as bgpvpn_rc_def
from neutron_lib.api.definitions import bgpvpn_vni as bgpvpn_vni_def
from neutron_lib.db import api as db_api
from neutron_lib.db import constants as db_const
from neutron_lib.db import model_base
from neutron_lib.db import model_query
from neutron_lib.db import standard_attr
from neutron_lib.db import utils as db_utils
from neutron_lib.plugins import directory

from networking_bgpvpn.neutron.extensions import bgpvpn as bgpvpn_ext
from networking_bgpvpn.neutron.extensions\
    import bgpvpn_routes_control as bgpvpn_rc_ext
from networking_bgpvpn.neutron.services.common import utils

LOG = log.getLogger(__name__)


class HasProjectNotNullable(model_base.HasProject):

    project_id = sa.Column(sa.String(db_const.PROJECT_ID_FIELD_SIZE),
                           index=True,
                           nullable=False)


class BGPVPNNetAssociation(standard_attr.HasStandardAttributes,
                           model_base.BASEV2, model_base.HasId,
                           HasProjectNotNullable):
    """Represents the association between a bgpvpn and a network."""
    __tablename__ = 'bgpvpn_network_associations'

    bgpvpn_id = sa.Column(sa.String(36),
                          sa.ForeignKey('bgpvpns.id', ondelete='CASCADE'),
                          nullable=False)
    network_id = sa.Column(sa.String(36),
                           sa.ForeignKey('networks.id', ondelete='CASCADE'),
                           nullable=False)
    sa.UniqueConstraint(bgpvpn_id, network_id)
    network = orm.relationship("Network",
                               backref=orm.backref('bgpvpn_associations',
                                                   cascade='all'),
                               lazy='joined',)

    # standard attributes support:
    api_collections = []
    api_sub_resources = [bgpvpn_def.NETWORK_ASSOCIATIONS]
    collection_resource_map = {bgpvpn_def.NETWORK_ASSOCIATIONS:
                               bgpvpn_def.NETWORK_ASSOCIATION}


class BGPVPNRouterAssociation(standard_attr.HasStandardAttributes,
                              model_base.BASEV2, model_base.HasId,
                              HasProjectNotNullable):
    """Represents the association between a bgpvpn and a router."""
    __tablename__ = 'bgpvpn_router_associations'

    bgpvpn_id = sa.Column(sa.String(36),
                          sa.ForeignKey('bgpvpns.id', ondelete='CASCADE'),
                          nullable=False)
    router_id = sa.Column(sa.String(36),
                          sa.ForeignKey('routers.id', ondelete='CASCADE'),
                          nullable=False)
    sa.UniqueConstraint(bgpvpn_id, router_id)
    advertise_extra_routes = sa.Column(sa.Boolean(), nullable=False,
                                       server_default=sa.true())
    router = orm.relationship("Router",
                              backref=orm.backref('bgpvpn_associations',
                                                  cascade='all'),
                              lazy='joined',)

    # standard attributes support:
    api_collections = []
    api_sub_resources = [bgpvpn_def.ROUTER_ASSOCIATIONS]
    collection_resource_map = {bgpvpn_def.ROUTER_ASSOCIATIONS:
                               bgpvpn_def.ROUTER_ASSOCIATION}


class BGPVPNPortAssociation(standard_attr.HasStandardAttributes,
                            model_base.BASEV2, model_base.HasId,
                            HasProjectNotNullable):
    """Represents the association between a bgpvpn and a port."""
    __tablename__ = 'bgpvpn_port_associations'

    bgpvpn_id = sa.Column(sa.String(36),
                          sa.ForeignKey('bgpvpns.id', ondelete='CASCADE'),
                          nullable=False)
    port_id = sa.Column(sa.String(36),
                        sa.ForeignKey('ports.id', ondelete='CASCADE'),
                        nullable=False)
    sa.UniqueConstraint(bgpvpn_id, port_id)
    advertise_fixed_ips = sa.Column(sa.Boolean(), nullable=False,
                                    server_default=sa.true())
    port = orm.relationship("Port",
                            backref=orm.backref('bgpvpn_associations',
                                                cascade='all'),
                            lazy='joined',)

    # standard attributes support:
    api_collections = []
    api_sub_resources = [bgpvpn_rc_def.PORT_ASSOCIATIONS]
    collection_resource_map = {bgpvpn_rc_def.PORT_ASSOCIATIONS:
                               bgpvpn_rc_def.PORT_ASSOCIATION}


class BGPVPN(standard_attr.HasStandardAttributes, model_base.BASEV2,
             model_base.HasId, model_base.HasProject):
    """Represents a BGPVPN Object."""
    name = sa.Column(sa.String(255))
    type = sa.Column(sa.Enum("l2", "l3",
                             name="vpn_types"),
                     nullable=False)
    route_targets = sa.Column(sa.String(255), nullable=False)
    import_targets = sa.Column(sa.String(255), nullable=True)
    export_targets = sa.Column(sa.String(255), nullable=True)
    route_distinguishers = sa.Column(sa.String(255), nullable=True)
    vni = sa.Column(sa.Integer, nullable=True)
    local_pref = sa.Column(sa.BigInteger, nullable=True)
    network_associations = orm.relationship("BGPVPNNetAssociation",
                                            backref="bgpvpn",
                                            lazy='select',
                                            cascade='all, delete-orphan')
    router_associations = orm.relationship("BGPVPNRouterAssociation",
                                           backref="bgpvpn",
                                           lazy='select',
                                           cascade='all, delete-orphan')
    port_associations = orm.relationship("BGPVPNPortAssociation",
                                         backref="bgpvpn",
                                         lazy='select',
                                         cascade='all, delete-orphan')

    # standard attributes support:
    api_collections = [bgpvpn_def.COLLECTION_NAME]
    collection_resource_map = {bgpvpn_def.COLLECTION_NAME:
                               bgpvpn_def.RESOURCE_NAME}


class BGPVPNPortAssociationRoute(model_base.BASEV2, model_base.HasId):
    """Represents an item of the 'routes' attribute of a port association."""
    __tablename__ = 'bgpvpn_port_association_routes'

    port_association_id = sa.Column(
        sa.String(length=36),
        sa.ForeignKey('bgpvpn_port_associations.id', ondelete='CASCADE'),
        nullable=False)
    type = sa.Column(sa.Enum(*bgpvpn_rc_def.ROUTE_TYPES,
                             name="bgpvpn_port_association_route_type"),
                     nullable=False)
    local_pref = sa.Column(sa.BigInteger(),
                           nullable=True)
    # prefix is NULL unless type is 'prefix'
    prefix = sa.Column(sa.String(49),
                       nullable=True)
    # bgpvpn_id is NULL unless type is 'bgpvpn'
    bgpvpn_id = sa.Column(sa.String(length=36),
                          sa.ForeignKey('bgpvpns.id', ondelete='CASCADE'),
                          nullable=True)

    port_association = orm.relationship(
        "BGPVPNPortAssociation",
        backref=orm.backref('routes',
                            cascade='all'),
        lazy='joined')
    bgpvpn = orm.relationship(
        "BGPVPN",
        backref=orm.backref("port_association_routes",
                            uselist=False,
                            lazy='select',
                            cascade='all, delete-orphan'),
        lazy='joined')


def _list_bgpvpns_result_filter_hook(query, filters):
    values = filters and filters.get('networks', [])
    if values:
        query = query.join(BGPVPNNetAssociation)
        query = query.filter(BGPVPNNetAssociation.network_id.in_(values))

    values = filters and filters.get('routers', [])
    if values:
        query = query.join(BGPVPNRouterAssociation)
        query = query.filter(BGPVPNRouterAssociation.router_id.in_(values))

    values = filters and filters.get('ports', [])
    if values:
        query = query.join(BGPVPNRouterAssociation)
        query = query.filter(BGPVPNPortAssociation.port_id.in_(values))

    return query


@db_api.CONTEXT_WRITER
def _add_port_assoc_route_db_from_dict(context, route, port_association_id):
    if route['type'] == 'prefix':
        kwargs = {'prefix': route['prefix']}
    elif route['type'] == 'bgpvpn':
        kwargs = {'bgpvpn_id': route['bgpvpn_id']}
    else:
        # not reached
        pass

    context.session.add(BGPVPNPortAssociationRoute(
        port_association_id=port_association_id,
        type=route['type'],
        local_pref=route.get('local_pref', None),
        **kwargs
    ))


def port_assoc_route_dict_from_db(route_db):
    route = {
        'type': route_db.type,
        'local_pref': route_db.local_pref
    }
    if route_db.type == 'prefix':
        route.update({'prefix': route_db.prefix})
    elif route_db.type == 'bgpvpn':
        route.update({'bgpvpn_id': route_db.bgpvpn_id})
    return route


class BGPVPNPluginDb():
    """BGPVPN service plugin database class using SQLAlchemy models."""

    def __new__(cls, *args, **kwargs):
        model_query.register_hook(
            BGPVPN,
            "bgpvpn_filter_by_resource_association",
            query_hook=None,
            filter_hook=None,
            result_filters=_list_bgpvpns_result_filter_hook)
        return super(BGPVPNPluginDb, cls).__new__(cls, *args, **kwargs)

    @db_api.CONTEXT_READER
    def _get_bgpvpns_for_tenant(self, session, tenant_id, fields):
        try:
            qry = session.query(BGPVPN)
            bgpvpns = qry.filter_by(tenant_id=tenant_id)
        except exc.NoResultFound:
            return

        return [self._make_bgpvpn_dict(bgpvpn, fields=fields)
                for bgpvpn in bgpvpns]

    @db_api.CONTEXT_READER
    def _make_bgpvpn_dict(self, bgpvpn_db, fields=None):
        net_list = [net_assocs.network_id for net_assocs in
                    bgpvpn_db.network_associations]
        router_list = [router_assocs.router_id for router_assocs in
                       bgpvpn_db.router_associations]
        port_list = [port_assocs.port_id for port_assocs in
                     bgpvpn_db.port_associations]
        res = {
            'id': bgpvpn_db['id'],
            'tenant_id': bgpvpn_db['tenant_id'],
            'networks': net_list,
            'routers': router_list,
            'ports': port_list,
            'name': bgpvpn_db['name'],
            'type': bgpvpn_db['type'],
            'route_targets':
                utils.rtrd_str2list(bgpvpn_db['route_targets']),
            'route_distinguishers':
                utils.rtrd_str2list(bgpvpn_db['route_distinguishers']),
            'import_targets':
                utils.rtrd_str2list(bgpvpn_db['import_targets']),
            'export_targets':
                utils.rtrd_str2list(bgpvpn_db['export_targets'])
        }

        plugin = directory.get_plugin(bgpvpn_def.ALIAS)
        if utils.is_extension_supported(plugin, bgpvpn_vni_def.ALIAS):
            res[bgpvpn_vni_def.VNI] = bgpvpn_db.get(bgpvpn_vni_def.VNI)
        if utils.is_extension_supported(plugin, bgpvpn_rc_def.ALIAS):
            res[bgpvpn_rc_def.LOCAL_PREF_KEY] = bgpvpn_db.get(
                bgpvpn_rc_def.LOCAL_PREF_KEY)

        return db_utils.resource_fields(res, fields)

    @db_api.CONTEXT_WRITER
    def create_bgpvpn(self, context, bgpvpn):
        rt = utils.rtrd_list2str(bgpvpn['route_targets'])
        i_rt = utils.rtrd_list2str(bgpvpn['import_targets'])
        e_rt = utils.rtrd_list2str(bgpvpn['export_targets'])
        rd = utils.rtrd_list2str(bgpvpn.get('route_distinguishers', ''))

        bgpvpn_db = BGPVPN(
            id=uuidutils.generate_uuid(),
            tenant_id=bgpvpn['tenant_id'],
            name=bgpvpn['name'],
            type=bgpvpn['type'],
            route_targets=rt,
            import_targets=i_rt,
            export_targets=e_rt,
            route_distinguishers=rd,
            vni=bgpvpn.get(bgpvpn_vni_def.VNI),
            local_pref=bgpvpn.get(bgpvpn_rc_def.LOCAL_PREF_KEY),
        )
        context.session.add(bgpvpn_db)
        return self._make_bgpvpn_dict(bgpvpn_db)

    @db_api.CONTEXT_READER
    def get_bgpvpns(self, context, filters=None, fields=None):
        return model_query.get_collection(
            context, BGPVPN, self._make_bgpvpn_dict,
            filters=filters, fields=fields)

    @db_api.CONTEXT_READER
    def _get_bgpvpn(self, context, id):
        try:
            return model_query.get_by_id(context, BGPVPN, id)
        except exc.NoResultFound:
            raise bgpvpn_ext.BGPVPNNotFound(id=id)

    @db_api.CONTEXT_READER
    def get_bgpvpn(self, context, id, fields=None):
        bgpvpn_db = self._get_bgpvpn(context, id)
        return self._make_bgpvpn_dict(bgpvpn_db, fields)

    @db_api.CONTEXT_WRITER
    def update_bgpvpn(self, context, id, bgpvpn):
        bgpvpn_db = self._get_bgpvpn(context, id)
        if bgpvpn:
            # Format Route Target lists to string
            if 'route_targets' in bgpvpn:
                rt = utils.rtrd_list2str(bgpvpn['route_targets'])
                bgpvpn['route_targets'] = rt
            if 'import_targets' in bgpvpn:
                i_rt = utils.rtrd_list2str(bgpvpn['import_targets'])
                bgpvpn['import_targets'] = i_rt
            if 'export_targets' in bgpvpn:
                e_rt = utils.rtrd_list2str(bgpvpn['export_targets'])
                bgpvpn['export_targets'] = e_rt
            if 'route_distinguishers' in bgpvpn:
                rd = utils.rtrd_list2str(bgpvpn['route_distinguishers'])
                bgpvpn['route_distinguishers'] = rd
            bgpvpn_db.update(bgpvpn)
        return self._make_bgpvpn_dict(bgpvpn_db)

    @db_api.CONTEXT_WRITER
    def delete_bgpvpn(self, context, id):
        bgpvpn_db = self._get_bgpvpn(context, id)
        bgpvpn = self._make_bgpvpn_dict(bgpvpn_db)
        context.session.delete(bgpvpn_db)
        return bgpvpn

    @db_api.CONTEXT_READER
    def _make_net_assoc_dict(self, net_assoc_db, fields=None):
        res = {'id': net_assoc_db['id'],
               'tenant_id': net_assoc_db['tenant_id'],
               'bgpvpn_id': net_assoc_db['bgpvpn_id'],
               'network_id': net_assoc_db['network_id']}
        return db_utils.resource_fields(res, fields)

    @db_api.CONTEXT_READER
    def _get_net_assoc(self, context, assoc_id, bgpvpn_id):
        try:
            query = model_query.query_with_hooks(context, BGPVPNNetAssociation)
            return query.filter(BGPVPNNetAssociation.id == assoc_id,
                                BGPVPNNetAssociation.bgpvpn_id == bgpvpn_id
                                ).one()
        except exc.NoResultFound:
            raise bgpvpn_ext.BGPVPNNetAssocNotFound(id=assoc_id,
                                                    bgpvpn_id=bgpvpn_id)

    def create_net_assoc(self, context, bgpvpn_id, net_assoc):
        try:
            with db_api.CONTEXT_WRITER.using(context):
                net_assoc_db = BGPVPNNetAssociation(
                    tenant_id=net_assoc['tenant_id'],
                    bgpvpn_id=bgpvpn_id,
                    network_id=net_assoc['network_id'])
                context.session.add(net_assoc_db)
            return self._make_net_assoc_dict(net_assoc_db)
        except db_exc.DBDuplicateEntry:
            LOG.warning("network %(net_id)s is already associated to "
                        "BGPVPN %(bgpvpn_id)s",
                        {'net_id': net_assoc['network_id'],
                         'bgpvpn_id': bgpvpn_id})
            raise bgpvpn_ext.BGPVPNNetAssocAlreadyExists(
                bgpvpn_id=bgpvpn_id, net_id=net_assoc['network_id'])

    @db_api.CONTEXT_READER
    def get_net_assoc(self, context, assoc_id, bgpvpn_id, fields=None):
        net_assoc_db = self._get_net_assoc(context, assoc_id, bgpvpn_id)
        return self._make_net_assoc_dict(net_assoc_db, fields)

    @db_api.CONTEXT_READER
    def get_net_assocs(self, context, bgpvpn_id, filters=None, fields=None):
        if not filters:
            filters = {}
        filters['bgpvpn_id'] = [bgpvpn_id]
        return model_query.get_collection(
            context, BGPVPNNetAssociation,
            self._make_net_assoc_dict,
            filters, fields)

    @db_api.CONTEXT_WRITER
    def delete_net_assoc(self, context, assoc_id, bgpvpn_id):
        LOG.info("deleting network association %(id)s for "
                 "BGPVPN %(bgpvpn)s", {'id': assoc_id,
                                       'bgpvpn': bgpvpn_id})
        net_assoc_db = self._get_net_assoc(context, assoc_id, bgpvpn_id)
        net_assoc = self._make_net_assoc_dict(net_assoc_db)
        context.session.delete(net_assoc_db)
        return net_assoc

    def _make_router_assoc_dict(self, router_assoc_db, fields=None):
        res = {'id': router_assoc_db['id'],
               'tenant_id': router_assoc_db['tenant_id'],
               'bgpvpn_id': router_assoc_db['bgpvpn_id'],
               'router_id': router_assoc_db['router_id'],
               'advertise_extra_routes': router_assoc_db[
                   'advertise_extra_routes']
               }
        return db_utils.resource_fields(res, fields)

    @db_api.CONTEXT_READER
    def _get_router_assoc(self, context, assoc_id, bgpvpn_id):
        try:
            query = model_query.query_with_hooks(
                context, BGPVPNRouterAssociation)
            return query.filter(BGPVPNRouterAssociation.id == assoc_id,
                                BGPVPNRouterAssociation.bgpvpn_id == bgpvpn_id
                                ).one()
        except exc.NoResultFound:
            raise bgpvpn_ext.BGPVPNRouterAssocNotFound(id=assoc_id,
                                                       bgpvpn_id=bgpvpn_id)

    @db_api.CONTEXT_WRITER
    def create_router_assoc(self, context, bgpvpn_id, router_association):
        router_id = router_association['router_id']
        try:
            router_assoc_db = BGPVPNRouterAssociation(
                tenant_id=router_association['tenant_id'],
                bgpvpn_id=bgpvpn_id,
                router_id=router_id)
            context.session.add(router_assoc_db)
            context.session.flush()
            return self._make_router_assoc_dict(router_assoc_db)
        except db_exc.DBDuplicateEntry:
            LOG.warning("router %(router_id)s is already associated to "
                        "BGPVPN %(bgpvpn_id)s",
                        {'router_id': router_id,
                         'bgpvpn_id': bgpvpn_id})
            raise bgpvpn_ext.BGPVPNRouterAssocAlreadyExists(
                bgpvpn_id=bgpvpn_id, router_id=router_association['router_id'])

    @db_api.CONTEXT_READER
    def get_router_assoc(self, context, assoc_id, bgpvpn_id, fields=None):
        router_assoc_db = self._get_router_assoc(context, assoc_id, bgpvpn_id)
        return self._make_router_assoc_dict(router_assoc_db, fields)

    @db_api.CONTEXT_READER
    def get_router_assocs(self, context, bgpvpn_id, filters=None, fields=None):
        if not filters:
            filters = {}
        filters['bgpvpn_id'] = [bgpvpn_id]
        return model_query.get_collection(
            context, BGPVPNRouterAssociation,
            self._make_router_assoc_dict,
            filters, fields)

    @db_api.CONTEXT_WRITER
    def update_router_assoc(self, context, assoc_id, bgpvpn_id, router_assoc):
        router_assoc_db = self._get_router_assoc(context,
                                                 assoc_id, bgpvpn_id)
        router_assoc_db.update(router_assoc)
        return self._make_router_assoc_dict(router_assoc_db)

    @db_api.CONTEXT_WRITER
    def delete_router_assoc(self, context, assoc_id, bgpvpn_id):
        LOG.info("deleting router association %(id)s for "
                 "BGPVPN %(bgpvpn)s",
                 {'id': assoc_id, 'bgpvpn': bgpvpn_id})
        router_assoc_db = self._get_router_assoc(context, assoc_id,
                                                 bgpvpn_id)
        router_assoc = self._make_router_assoc_dict(router_assoc_db)
        context.session.delete(router_assoc_db)
        return router_assoc

    @db_api.CONTEXT_READER
    def _make_port_assoc_dict(self, port_assoc_db, fields=None):
        routes = [port_assoc_route_dict_from_db(r)
                  for r in port_assoc_db['routes']]
        res = {'id': port_assoc_db['id'],
               'tenant_id': port_assoc_db['tenant_id'],
               'bgpvpn_id': port_assoc_db['bgpvpn_id'],
               'port_id': port_assoc_db['port_id'],
               'advertise_fixed_ips': port_assoc_db['advertise_fixed_ips'],
               'routes': routes}
        return db_utils.resource_fields(res, fields)

    @db_api.CONTEXT_READER
    def _get_port_assoc(self, context, assoc_id, bgpvpn_id):
        try:
            query = model_query.query_with_hooks(
                context, BGPVPNPortAssociation)
            return query.filter(BGPVPNPortAssociation.id == assoc_id,
                                BGPVPNPortAssociation.bgpvpn_id == bgpvpn_id
                                ).one()
        except exc.NoResultFound:
            raise bgpvpn_rc_ext.BGPVPNPortAssocNotFound(id=assoc_id,
                                                        bgpvpn_id=bgpvpn_id)

    def create_port_assoc(self, context, bgpvpn_id, port_association):
        port_id = port_association['port_id']
        advertise_fixed_ips = port_association['advertise_fixed_ips']
        try:
            with db_api.CONTEXT_WRITER.using(context):
                port_assoc_db = BGPVPNPortAssociation(
                    tenant_id=port_association['tenant_id'],
                    bgpvpn_id=bgpvpn_id,
                    port_id=port_id,
                    advertise_fixed_ips=advertise_fixed_ips)
                context.session.add(port_assoc_db)
        except db_exc.DBDuplicateEntry:
            LOG.warning(("port %(port_id)s is already associated to "
                         "BGPVPN %(bgpvpn_id)s"),
                        {'port_id': port_id,
                         'bgpvpn_id': bgpvpn_id})
            raise bgpvpn_rc_ext.BGPVPNPortAssocAlreadyExists(
                bgpvpn_id=bgpvpn_id, port_id=port_association['port_id'])

        for route in port_association['routes']:
            _add_port_assoc_route_db_from_dict(
                context, route, port_assoc_db.id)
        return self._make_port_assoc_dict(port_assoc_db)

    @db_api.CONTEXT_READER
    def get_port_assoc(self, context, assoc_id, bgpvpn_id, fields=None):
        port_assoc_db = self._get_port_assoc(context, assoc_id, bgpvpn_id)
        return self._make_port_assoc_dict(port_assoc_db, fields)

    @db_api.CONTEXT_READER
    def get_port_assocs(self, context, bgpvpn_id, filters=None, fields=None):
        if not filters:
            filters = {}
        filters['bgpvpn_id'] = [bgpvpn_id]
        return model_query.get_collection(
            context, BGPVPNPortAssociation,
            self._make_port_assoc_dict,
            filters, fields)

    def update_port_assoc(self, context, assoc_id, bgpvpn_id, port_assoc):
        with db_api.CONTEXT_WRITER.using(context):
            port_assoc_db = self._get_port_assoc(context, assoc_id, bgpvpn_id)
            for route_db in port_assoc_db.routes:
                context.session.delete(route_db)
            for route in port_assoc.pop('routes', []):
                _add_port_assoc_route_db_from_dict(context, route, assoc_id)
            port_assoc_db.update(port_assoc)
        context.session.refresh(port_assoc_db)
        return self._make_port_assoc_dict(port_assoc_db)

    @db_api.CONTEXT_WRITER
    def delete_port_assoc(self, context, assoc_id, bgpvpn_id):
        LOG.info(("deleting port association %(id)s for "
                  "BGPVPN %(bgpvpn)s"),
                 {'id': assoc_id, 'bgpvpn': bgpvpn_id})
        port_assoc_db = self._get_port_assoc(context, assoc_id,
                                             bgpvpn_id)
        port_assoc = self._make_port_assoc_dict(port_assoc_db)
        context.session.delete(port_assoc_db)
        return port_assoc



# Copyright (c) 2017 Orange.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from collections import namedtuple

from bgpvpn_dashboard.api import bgpvpn as bgpvpn_api
from bgpvpn_dashboard.dashboards.admin.bgpvpn import tables as bgpvpn_tables
from bgpvpn_dashboard.dashboards.admin.bgpvpn import views as bgpvpn_views

from openstack_dashboard.test import helpers

VIEWS = "bgpvpn_dashboard.dashboards.admin.bgpvpn.views"


class TestIndexView(helpers.APITestCase):

    def setUp(self):
        super(TestIndexView, self).setUp()
        mock_request = mock.Mock(horizon={'async_messages': []})
        self.bgpvpn_view = bgpvpn_views.IndexView(request=mock_request)
        self.assertEqual(bgpvpn_tables.BgpvpnTable,
                         self.bgpvpn_view.table_class)

    def _get_mock_bgpvpn(self, prefix):
        bgpvpn_info = {}
        if prefix:
            bgpvpn_info = {
                "name": "%s_name" % prefix,
                "route_targets": [],
                "import_targets": [],
                "export_targets": [],
                "networks": [],
                "routers": [],
                "tenant_id": "tenant_id",
                "type": "l3"
            }
        return bgpvpn_api.Bgpvpn(bgpvpn_info)

    @mock.patch.object(bgpvpn_views.api, 'keystone', autospec=True)
    def test_get_tenant_name(self, mock_api):
        Tenant = namedtuple("Tenant", ["id", "name"])
        tenant = Tenant("tenant_id", "tenant_name")

        mock_api.tenant_get.return_value = tenant
        result = self.bgpvpn_view._get_tenant_name("tenant_id")

        mock_api.tenant_get.assert_called_once_with(
            self.bgpvpn_view.request, "tenant_id")
        self.assertEqual(result, "tenant_name")

    @mock.patch('%s.IndexView._get_tenant_name' % VIEWS,
                return_value={"tenant_id": "tenant_name"})
    @mock.patch.object(bgpvpn_views, 'api', autospec=True)
    @mock.patch.object(bgpvpn_views, 'bgpvpn_api', autospec=True)
    def test_get_data(self, mock_bgpvpn_api, mock_api, mock_get_tenant_name):
        bgpvpn_foo = self._get_mock_bgpvpn("foo")
        bgpvpn_bar = self._get_mock_bgpvpn("bar")
        mock_neutron_client = mock_api.neutron.neutronclient(mock.Mock())
        mock_bgpvpn_api.bgpvpns_list.return_value = [bgpvpn_foo, bgpvpn_bar]
        mock_neutron_client.list_networks.return_value = []
        mock_neutron_client.list_routers.return_value = []

        expected_bgpvpns = [bgpvpn_foo, bgpvpn_bar]
        result = self.bgpvpn_view.get_data()

        calls = [mock.call("tenant_id"), mock.call("tenant_id")]
        mock_get_tenant_name.assert_has_calls(calls)
        self.assertEqual(result, expected_bgpvpns)



# Copyright (c) 2017 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import tabs

from bgpvpn_dashboard.api import bgpvpn as bgpvpn_api

from bgpvpn_dashboard.dashboards.project.bgpvpn.network_associations \
    import tabs as project_tabs


class DetailView(tabs.TabView):
    tab_group_class = project_tabs.NetworkAssociationDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ network_association.id }}"

    def get_data(self):
        network_association_id = self.kwargs['network_association_id']
        bgpvpn_id = self.kwargs['bgpvpn_id']
        try:
            network_association = bgpvpn_api.network_association_get(
                self.request, bgpvpn_id, network_association_id)
            return network_association
        except Exception:
            network_association = []
            msg = _('Unable to retrieve network association details.')
            exceptions.handle(self.request, msg,
                              redirect=self.get_redirect_url())
        return network_association

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        network_association = self.get_data()
        context["url"] = reverse(
            "horizon:project:bgpvpn:network_associations_tab",
            args=(self.kwargs["bgpvpn_id"], network_association.id))
        return context

    def get_tabs(self, request, *args, **kwargs):
        network_association = self.get_data()
        return self.tab_group_class(
            request, network_association=network_association, **kwargs)

    @staticmethod
    def get_redirect_url():
        return reverse('horizon:project:bgpvpn:index')



# Copyright (c) 2016 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import logging

from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import workflows
from openstack_dashboard import api

from bgpvpn_dashboard.api import bgpvpn as bgpvpn_api

LOG = logging.getLogger(__name__)


class AddRouterParametersInfoAction(workflows.Action):

    advertise_extra_routes = forms.BooleanField(
        label=_("Advertise Extra Routes"),
        initial=True,
        required=False,
        help_text="Boolean flag controlling whether or not the routes "
                  "specified in the routes attribute of the router will be "
                  "advertised to the BGPVPN (default: true).")

    class Meta(object):
        name = _("Optional Parameters")
        slug = "add_router_parameters"

    def __init__(self, request, context, *args, **kwargs):
        super(AddRouterParametersInfoAction, self).__init__(
            request, context, *args, **kwargs)
        if 'with_parameters' in context:
            self.fields['with_parameters'] = forms.BooleanField(
                initial=context['with_parameters'],
                required=False,
                widget=forms.HiddenInput()
            )


class CreateRouterAssociationInfoAction(workflows.Action):

    router_resource = forms.ChoiceField(
        label=_("Associate Router"),
        widget=forms.ThemableSelectWidget(
            data_attrs=('name', 'id'),
            transform=lambda x: "%s" % x.name_or_id))

    class Meta(object):
        name = _("Create Association")
        help_text = _("Create a new router association.")
        slug = "create_router_association"

    def __init__(self, request, context, *args, **kwargs):
        super(CreateRouterAssociationInfoAction, self).__init__(
            request, context, *args, **kwargs)

        # when an admin user uses the project panel BGPVPN, there is no
        # tenant_id in context because bgpvpn_get doesn't return it
        if request.user.is_superuser and context.get("project_id"):
            tenant_id = context.get("project_id")
        else:
            tenant_id = self.request.user.tenant_id

        try:
            routers = api.neutron.router_list(request, tenant_id=tenant_id)
            if routers:
                choices = [('', _("Choose a router"))] + [(r.id, r) for r in
                                                          routers]
                self.fields['router_resource'].choices = choices
            else:
                self.fields['router_resource'].choices = [('', _("No router"))]
        except Exception:
            exceptions.handle(request, _("Unable to retrieve routers"))

        if api.neutron.is_extension_supported(request,
                                              'bgpvpn-routes-control'):
            self.fields['with_parameters'] = forms.BooleanField(
                label=_("Optional parameters"),
                initial=False,
                required=False,
                widget=forms.CheckboxInput(attrs={
                    'class': 'switchable',
                    'data-hide-tab': 'router_association__'
                                     'add_router_parameters',
                    'data-hide-on-checked': 'false'
                }))


class AddRouterParametersInfo(workflows.Step):
    action_class = AddRouterParametersInfoAction
    depends_on = ("bgpvpn_id", "name")
    contributes = ("advertise_extra_routes",)


class CreateRouterAssociationInfo(workflows.Step):
    action_class = CreateRouterAssociationInfoAction
    contributes = ("router_resource", "with_parameters")


class RouterAssociation(workflows.Workflow):
    slug = "router_association"
    name = _("Associate a BGPVPN to a Router")
    finalize_button_name = _("Create")
    success_message = _('Router association with "%s" created.')
    failure_message = _('Unable to create a router association with "%s".')
    success_url = "horizon:project:bgpvpn:index"
    default_steps = (CreateRouterAssociationInfo,
                     AddRouterParametersInfo)
    wizard = True

    def format_status_message(self, message):
        name = self.context['name'] or self.context['bgpvpn_id']
        return message % name

    def handle(self, request, context):
        bgpvpn_id = context['bgpvpn_id']
        router_id = context["router_resource"]
        msg_error = _("Unable to associate router %s") % router_id
        try:
            router_association = bgpvpn_api.router_association_create(
                request, bgpvpn_id, router_id=router_id)
        except Exception:
            exceptions.handle(request, msg_error)
            return False
        if not context["with_parameters"]:
            return True
        asso_id = router_association['router_association']['id']
        try:
            bgpvpn_api.router_association_update(
                request, bgpvpn_id, asso_id,
                advertise_extra_routes=context['advertise_extra_routes'])
            return True
        except exceptions:
            bgpvpn_api.router_association_delete(request, asso_id, bgpvpn_id)
            exceptions.handle(request, msg_error)
            return False



# Copyright (c) 2017 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils import safestring
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import exceptions
from horizon import tables

from openstack_dashboard import api

from bgpvpn_dashboard.api import bgpvpn as bgpvpn_api

LOG = logging.getLogger(__name__)


class DeleteRouterAssociation(tables.DeleteAction):

    @staticmethod
    def action_present(count):
        return ungettext_lazy(u"Delete Router Association",
                              u"Delete Router Associations",
                              count)

    @staticmethod
    def action_past(count):
        return ungettext_lazy(u"Deleted Router Association",
                              u"Deleted Router Associations",
                              count)

    def delete(self, request, asso_id):
        try:
            bgpvpn_api.router_association_delete(
                request, asso_id, self.table.kwargs['bgpvpn_id'])
        except Exception:
            msg = _('Failed to delete Router Association %s') % asso_id
            LOG.info(msg)
            redirect = reverse('horizon:project:bgpvpn:detail')
            exceptions.handle(request, msg, redirect=redirect)


class UpdateRouterAssociation(tables.LinkAction):
    name = "update"
    verbose_name = _("Update Router Association")
    url = "horizon:project:bgpvpn:update-router-association"
    classes = ("ajax-modal",)
    icon = "pencil"

    def get_link_url(self, asso):
        bgpvpn_id = self.table.kwargs['bgpvpn_id']
        return reverse(self.url, args=(bgpvpn_id, asso.id))

    def allowed(self, request, datum=None):
        if api.neutron.is_extension_supported(request,
                                              'bgpvpn-routes-control'):
            return True


class CreateRouterAssociation(tables.LinkAction):
    name = "create_router_association"
    verbose_name = _("Create Router Association")
    url = "horizon:project:bgpvpn:create-router-association"
    classes = ("ajax-modal",)
    icon = "pencil"

    def get_link_url(self, datum=None):
        bgpvpn_id = self.table.kwargs['bgpvpn_id']
        return reverse(self.url, args=(bgpvpn_id,))


class IDColumn(tables.Column):
    url = "horizon:project:bgpvpn:router_assos:detail"

    def get_link_url(self, asso):
        bgpvpn_id = self.table.kwargs['bgpvpn_id']
        return reverse(self.url, args=(bgpvpn_id, asso.id))


class RouterColumn(tables.Column):
    def get_raw_data(self, asso):
        url = reverse('horizon:project:routers:detail', args=[asso.router_id])
        instance = '<a href=%s>%s</a>' % (url,
                                          asso.router_name or asso.router_id)
        return safestring.mark_safe(instance)


class RouterAssociationsTable(tables.DataTable):
    id = IDColumn("id", verbose_name=_("Association ID"),
                  link='horizon:project:bgpvpn:router_assos:detail')
    router = RouterColumn("router_id", verbose_name=_("Router"))
    failure_url = reverse_lazy('horizon:project:bgpvpn:detail')

    class Meta(object):
        name = "router_associations"
        verbose_name = _("Router Associations")
        table_actions = (CreateRouterAssociation, DeleteRouterAssociation)
        row_actions = (UpdateRouterAssociation, DeleteRouterAssociation,)
        hidden_title = False

    def get_object_display(self, asso):
        return asso.id



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from oslo_policy import policy

from networking_bgpvpn.policies import base


rules = [
    policy.DocumentedRuleDefault(
        'create_bgpvpn',
        base.RULE_ADMIN_ONLY,
        'Create a BGP VPN',
        [
            {
                'method': 'POST',
                'path': '/bgpvpn/bgpvpns',
            },
        ]
    ),

    policy.DocumentedRuleDefault(
        'update_bgpvpn',
        base.RULE_ADMIN_OR_OWNER,
        'Update a BGP VPN',
        [
            {
                'method': 'PUT',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
    # TODO(amotoki): tenant_id is not updatable, so perhaps this can be dropped
    policy.DocumentedRuleDefault(
        'update_bgpvpn:tenant_id',
        base.RULE_ADMIN_ONLY,
        'Update ``tenant_id`` attribute of a BGP VPN',
        [
            {
                'method': 'PUT',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
    policy.DocumentedRuleDefault(
        'update_bgpvpn:route_targets',
        base.RULE_ADMIN_ONLY,
        'Update ``route_targets`` attribute of a BGP VPN',
        [
            {
                'method': 'PUT',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
    policy.DocumentedRuleDefault(
        'update_bgpvpn:import_targets',
        base.RULE_ADMIN_ONLY,
        'Update ``import_targets`` attribute of a BGP VPN',
        [
            {
                'method': 'PUT',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
    policy.DocumentedRuleDefault(
        'update_bgpvpn:export_targets',
        base.RULE_ADMIN_ONLY,
        'Update ``export_targets`` attribute of a BGP VPN',
        [
            {
                'method': 'PUT',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
    policy.DocumentedRuleDefault(
        'update_bgpvpn:route_distinguishers',
        base.RULE_ADMIN_ONLY,
        'Update ``route_distinguishers`` attribute of a BGP VPN',
        [
            {
                'method': 'PUT',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
    # TODO(amotoki): vni is not updatable, so perhaps this can be dropped
    policy.DocumentedRuleDefault(
        'update_bgpvpn:vni',
        base.RULE_ADMIN_ONLY,
        'Update ``vni`` attribute of a BGP VPN',
        [
            {
                'method': 'PUT',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),

    policy.DocumentedRuleDefault(
        'delete_bgpvpn',
        base.RULE_ADMIN_ONLY,
        'Delete a BGP VPN',
        [
            {
                'method': 'DELETE',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
    policy.DocumentedRuleDefault(
        'get_bgpvpn',
        base.RULE_ADMIN_OR_OWNER,
        'Get BGP VPNs',
        [
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns',
            },
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),

    policy.DocumentedRuleDefault(
        'get_bgpvpn:tenant_id',
        base.RULE_ADMIN_ONLY,
        'Get ``tenant_id`` attributes of BGP VPNs',
        [
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns',
            },
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
    policy.DocumentedRuleDefault(
        'get_bgpvpn:route_targets',
        base.RULE_ADMIN_ONLY,
        'Get ``route_targets`` attributes of BGP VPNs',
        [
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns',
            },
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
    policy.DocumentedRuleDefault(
        'get_bgpvpn:import_targets',
        base.RULE_ADMIN_ONLY,
        'Get ``import_targets`` attributes of BGP VPNs',
        [
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns',
            },
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
    policy.DocumentedRuleDefault(
        'get_bgpvpn:export_targets',
        base.RULE_ADMIN_ONLY,
        'Get ``export_targets`` attributes of  BGP VPNs',
        [
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns',
            },
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
    policy.DocumentedRuleDefault(
        'get_bgpvpn:route_distinguishers',
        base.RULE_ADMIN_ONLY,
        'Get ``route_distinguishers`` attributes of BGP VPNs',
        [
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns',
            },
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
    policy.DocumentedRuleDefault(
        'get_bgpvpn:vni',
        base.RULE_ADMIN_ONLY,
        'Get ``vni`` attributes of BGP VPNs',
        [
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns',
            },
            {
                'method': 'GET',
                'path': '/bgpvpn/bgpvpns/{id}',
            },
        ]
    ),
]


def list_rules():
    return rules



# Copyright (c) 2015 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sqlalchemy import orm

from neutron.api.rpc.callbacks import events as rpc_events
from neutron.api.rpc.handlers import resources_rpc
from neutron.db.models import external_net

from neutron_lib.api.definitions import bgpvpn_routes_control as bgpvpn_rc_def
from neutron_lib.api.definitions import bgpvpn_vni as bgpvpn_vni_def
from neutron_lib.callbacks import events
from neutron_lib.callbacks import registry
from neutron_lib.callbacks import resources
from neutron_lib.db import api as db_api
from neutron_lib import exceptions as n_exc
from neutron_lib.plugins import directory

from networking_bagpipe.objects import bgpvpn as bgpvpn_objects

from oslo_log import helpers as log_helpers
from oslo_log import log as logging

from networking_bgpvpn.neutron.extensions import bgpvpn as bgpvpn_ext
from networking_bgpvpn.neutron.services.common import utils
from networking_bgpvpn.neutron.services.service_drivers import driver_api


LOG = logging.getLogger(__name__)


BAGPIPE_DRIVER_NAME = "bagpipe"


class BGPVPNExternalNetAssociation(n_exc.NeutronException):
    message = _("driver does not support associating an external"
                "network to a BGPVPN")


@db_api.CONTEXT_READER
def network_is_external(context, net_id):
    try:
        context.session.query(external_net.ExternalNetwork).filter_by(
            network_id=net_id).one()
        return True
    except orm.exc.NoResultFound:
        return False


def _log_callback_processing_exception(resource, event, trigger, metadata, e):
    LOG.exception("Error during notification processing "
                  "%(resource)s %(event)s, %(trigger)s, "
                  "%(metadata)s: %(exc)s",
                  {'trigger': trigger,
                   'resource': resource,
                   'event': event,
                   'metadata': metadata,
                   'exc': e})


@registry.has_registry_receivers
class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriverRC):

    """BGPVPN Service Driver class for BaGPipe"""

    more_supported_extension_aliases = [bgpvpn_rc_def.ALIAS,
                                        bgpvpn_vni_def.ALIAS]

    def __init__(self, service_plugin):
        super(BaGPipeBGPVPNDriver, self).__init__(service_plugin)

        self._push_rpc = resources_rpc.ResourcesPushRpcApi()

    def _push_association(self, context, association, event_type):
        self._push_associations(context, [association], event_type)

    def _push_associations(self, context, associations, event_type):
        if not associations:
            return
        for assoc in associations:
            LOG.debug("pushing %s %s (%s)", event_type, assoc, assoc.bgpvpn)
        self._push_rpc.push(context, associations, event_type)

    def _common_precommit_checks(self, bgpvpn):
        # No support yet for specifying route distinguishers
        if bgpvpn.get('route_distinguishers', None):
            raise bgpvpn_ext.BGPVPNRDNotSupported(driver=BAGPIPE_DRIVER_NAME)

    def create_bgpvpn_precommit(self, context, bgpvpn):
        self._common_precommit_checks(bgpvpn)

    def delete_bgpvpn_precommit(self, context, bgpvpn):
        self._push_bgpvpn_associations(context, bgpvpn['id'],
                                       rpc_events.DELETED)

    def update_bgpvpn_precommit(self, context, old_bgpvpn, bgpvpn):
        self._common_precommit_checks(bgpvpn)

    def update_bgpvpn_postcommit(self, context, old_bgpvpn, bgpvpn):
        (added_keys, removed_keys, changed_keys) = (
            utils.get_bgpvpn_differences(bgpvpn, old_bgpvpn))
        ATTRIBUTES_TO_IGNORE = set(['name'])
        moving_keys = added_keys | removed_keys | changed_keys
        if len(moving_keys ^ ATTRIBUTES_TO_IGNORE):
            self._push_bgpvpn_associations(context, bgpvpn['id'],
                                           rpc_events.UPDATED)

    def _push_bgpvpn_associations(self, context, bgpvpn_id, event_type):
        self._push_associations(
            context,
            (bgpvpn_objects.BGPVPNNetAssociation.get_objects(
                context,
                bgpvpn_id=bgpvpn_id) +
             bgpvpn_objects.BGPVPNRouterAssociation.get_objects(
                context,
                bgpvpn_id=bgpvpn_id)
             ),
            event_type)

    def create_net_assoc_precommit(self, context, net_assoc):
        if network_is_external(context, net_assoc['network_id']):
            raise BGPVPNExternalNetAssociation()

    def create_net_assoc_postcommit(self, context, net_assoc):
        self._push_association(
            context,
            bgpvpn_objects.BGPVPNNetAssociation.get_object(
                context,
                id=net_assoc['id']),
            rpc_events.CREATED)

    def delete_net_assoc_precommit(self, context, net_assoc):
        self._push_association(
            context,
            bgpvpn_objects.BGPVPNNetAssociation.get_object(
                context,
                id=net_assoc['id']),
            rpc_events.DELETED)

    def create_port_assoc_postcommit(self, context, port_assoc):
        self._push_association(
            context,
            bgpvpn_objects.BGPVPNPortAssociation.get_object(
                context,
                id=port_assoc['id']),
            rpc_events.CREATED)

    def update_port_assoc_postcommit(self, context,
                                     old_port_assoc, port_assoc):
        self._push_association(
            context,
            bgpvpn_objects.BGPVPNPortAssociation.get_object(
                context,
                id=port_assoc['id']),
            rpc_events.UPDATED)

    def delete_port_assoc_precommit(self, context, port_assoc):
        self._push_association(
            context,
            bgpvpn_objects.BGPVPNPortAssociation.get_object(
                context,
                id=port_assoc['id']),
            rpc_events.DELETED)

    def create_router_assoc_postcommit(self, context, router_assoc):
        self._push_association(
            context,
            bgpvpn_objects.BGPVPNRouterAssociation.get_object(
                context,
                id=router_assoc['id']),
            rpc_events.CREATED)

    def delete_router_assoc_precommit(self, context, router_assoc):
        self._push_association(
            context,
            bgpvpn_objects.BGPVPNRouterAssociation.get_object(
                context,
                id=router_assoc['id']),
            rpc_events.DELETED)

    @log_helpers.log_method_call
    def notify_router_interface_created(self, context, router_id, net_id):
        # update associations for the networks on which the router was plugged
        self._push_associations(
            context,
            (bgpvpn_objects.BGPVPNNetAssociation.get_objects(
                context,
                network_id=net_id) +
             bgpvpn_objects.BGPVPNRouterAssociation.get_objects(
                context,
                network_id=net_id)),
            rpc_events.UPDATED)

    @log_helpers.log_method_call
    def notify_router_interface_deleted(self, context, router_id, net_id):
        # update associations for the networks on which the router was plugged
        associations = (
            bgpvpn_objects.BGPVPNNetAssociation.get_objects(
                context,
                network_id=net_id) +
            bgpvpn_objects.BGPVPNRouterAssociation.get_objects(
                context,
                router_id=router_id)
        )

        # NOTE(tmorin): the gateway_mac information in these notifications
        # will not be None, as it should, because unfortunately the OVObjects
        # are created before the DB is updated after interface removal.
        # So we reprocess them to empty this field...
        for assoc in associations:
            for subnet in assoc.all_subnets(net_id):
                subnet['gateway_mac'] = None

        self._push_associations(context, associations, rpc_events.UPDATED)

    @registry.receives(resources.ROUTER_INTERFACE, [events.AFTER_CREATE])
    @log_helpers.log_method_call
    def registry_router_interface_created(self, resource, event, trigger,
                                          payload=None):
        try:
            context = payload.context
            router_id = payload.resource_id
            net_id = payload.metadata.get('port')['network_id']
            self.notify_router_interface_created(context, router_id, net_id)
        except Exception as e:
            _log_callback_processing_exception(resource, event, trigger,
                                               payload.metadata, e)

    # need to subscribe to router interface *before*_delete
    # because after delete, we can't build the OVO objects from the DB anymore
    @registry.receives(resources.ROUTER_INTERFACE, [events.BEFORE_DELETE])
    @log_helpers.log_method_call
    def registry_router_interface_deleted(self, resource, event, trigger,
                                          payload=None):
        try:
            context = payload.context
            # for router_interface after_delete, in stable/newton, the
            # callback does not include the router_id directly, but we find
            # it in the port device_id
            router_id = payload.resource_id
            subnet_id = payload.metadata['subnet_id']
            # find the network for this subnet
            network_id = directory.get_plugin().get_subnet(
                context, subnet_id)['network_id']
            self.notify_router_interface_deleted(
                context, router_id, network_id)
        except Exception as e:
            _log_callback_processing_exception(
                resource, event, trigger,
                {'subnet_id': subnet_id, 'router_id': router_id}, e)



#
# Copyright 2017 Ericsson India Global Services Pvt Ltd. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

import copy
from unittest import mock

from oslo_utils import uuidutils

from neutron.tests.unit.api.v2 import test_base
from neutron.tests.unit.extensions import base as test_extensions_base
from neutron_lib.api.definitions import bgpvpn as bgpvpn_def
from neutron_lib.api.definitions import bgpvpn_vni as bgpvpn_vni_def
from webob import exc

from networking_bgpvpn.neutron.extensions import bgpvpn

_uuid = uuidutils.generate_uuid
_get_path = test_base._get_path
BGPVPN_PREFIX = 'bgpvpn'
BGPVPN_URI = BGPVPN_PREFIX + '/' + 'bgpvpns'
BGPVPN_PLUGIN_BASE_NAME = (bgpvpn.BGPVPNPluginBase.__module__ +
                           '.' +
                           bgpvpn.BGPVPNPluginBase.__name__)


class BgpvpnVniTestExtensionManager(object):

    def get_resources(self):
        bgpvpn_def.RESOURCE_ATTRIBUTE_MAP[bgpvpn_def.COLLECTION_NAME].update(
            bgpvpn_vni_def.RESOURCE_ATTRIBUTE_MAP[bgpvpn_def.COLLECTION_NAME])
        return bgpvpn.Bgpvpn.get_resources()

    def get_actions(self):
        return []

    def get_request_extensions(self):
        return []


class BgpvpnVniExtensionTestCase(test_extensions_base.ExtensionTestCase):
    fmt = 'json'

    def setUp(self):
        super(BgpvpnVniExtensionTestCase, self).setUp()
        plural_mappings = {'bgpvpn': 'bgpvpns'}
        self.setup_extension(
            BGPVPN_PLUGIN_BASE_NAME,
            bgpvpn_def.ALIAS,
            BgpvpnVniTestExtensionManager(),
            BGPVPN_PREFIX,
            plural_mappings=plural_mappings,
            translate_resource_name=True)
        self.instance = self.plugin.return_value

    def test_bgpvpn_create(self):
        bgpvpn_id = _uuid()
        data = {
            'bgpvpn': {'name': 'bgpvpn1',
                       'type': 'l3',
                       'route_targets': ['1234:56'],
                       'vni': 1000,
                       'tenant_id': _uuid()}
        }
        expected_ret_val = copy.copy(data['bgpvpn'])
        expected_ret_val['import_targets'] = []
        expected_ret_val['export_targets'] = []
        expected_ret_val['route_distinguishers'] = []
        expected_ret_val['vni'] = 1000
        expected_call_args = copy.copy(expected_ret_val)
        expected_ret_val.update({'id': bgpvpn_id})

        self.instance.create_bgpvpn.return_value = expected_ret_val
        res = self.api.post(_get_path(BGPVPN_URI, fmt=self.fmt),
                            self.serialize(data),
                            content_type='application/%s' % self.fmt)
        self.assertTrue(self.instance.create_bgpvpn.called)
        self.assertDictSupersetOf(
            expected_call_args,
            self.instance.create_bgpvpn.call_args[1]['bgpvpn']['bgpvpn'])
        self.assertEqual(res.status_int, exc.HTTPCreated.code)
        res = self.deserialize(res)
        self.assertIn('bgpvpn', res)
        self.assertDictSupersetOf(expected_ret_val, res['bgpvpn'])

    def test_bgpvpn_get(self):
        bgpvpn_id = _uuid()
        return_value = {'name': 'bgpvpn1',
                        'type': 'l3',
                        'route_targets': ['1234:56'],
                        'tenant_id': _uuid(),
                        'vni': 1000,
                        'id': bgpvpn_id}

        self.instance.get_bgpvpn.return_value = return_value

        res = self.api.get(_get_path(BGPVPN_URI,
                                     id=bgpvpn_id,
                                     fmt=self.fmt))

        self.instance.get_bgpvpn.assert_called_with(
            mock.ANY, bgpvpn_id, fields=mock.ANY
        )
        self.assertEqual(res.status_int, exc.HTTPOk.code)
        res = self.deserialize(res)
        self.assertIn('bgpvpn', res)
        self.assertEqual(res['bgpvpn'], return_value)

    def test_bgpvpn_delete(self):
        self._test_entity_delete('bgpvpn')



# Copyright (c) 2017 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django import test
from unittest import mock

from bgpvpn_dashboard.dashboards.admin.bgpvpn import tables as bgpvpn_tables


class TestDeleteBgpvpns(test.TestCase):
    @mock.patch.object(bgpvpn_tables, 'bgpvpn_api')
    def test_delete(self, mock_bgpvpn_api):
        mock_request = mock.Mock()
        delete_bgpvpn = bgpvpn_tables.DeleteBgpvpn()
        delete_bgpvpn.delete(mock_request, "id")
        mock_bgpvpn_api.bgpvpn_delete.assert_called_once_with(mock_request,
                                                              "id")



# Copyright (c) 2016 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from openstack_dashboard.test import helpers as test

from bgpvpn_dashboard import api
from bgpvpn_dashboard.test import helpers as bgpvpn_test

from neutronclient.v2_0.client import Client as neutronclient


class BgpvpnApiTests(bgpvpn_test.APITestCase):

    def setUp(self):
        bgpvpn_test.APITestCase.setUp(self)

        # Since the early days of networking-bgpvpn, the package was providing
        # a neutronclient.extensions entry point so that neutronclient would
        # dynamically add BGPVPN API methods to a Client instance.  This is
        # only kept for backward compatibility, but not required anymore for
        # users of >=Ocata python-neutronclient. However, the dynamic addition
        # of these methods makes is a pain to mock. The patch below disables
        # the addition of the dynamic BGPVPN methods.
        mock.patch('neutronclient.v2_0.client.Client._register_extensions'
                   ).start()

    @test.create_mocks({neutronclient: ('list_bgpvpns',)})
    def test_bgpvpn_list(self):
        exp_bgpvpns = self.bgpvpns.list()

        api_bgpvpns = {'bgpvpns': self.api_bgpvpns.list()}
        self.mock_list_bgpvpns.return_value = api_bgpvpns

        ret_vals = api.bgpvpn.bgpvpns_list(self.request)
        for (ret_val, exp_bgpvpn) in zip(ret_vals, exp_bgpvpns):
            self.assertIsInstance(ret_val, api.bgpvpn.Bgpvpn)
            self.assertEqual(exp_bgpvpn.id, ret_val.id)
            self.assertEqual(exp_bgpvpn.name, ret_val.name)

    @test.create_mocks({neutronclient: ('create_bgpvpn',)})
    def test_bgpvpn_create(self):
        bgpvpn = self.bgpvpns.first()
        data = {'name': bgpvpn.name,
                'route_targets': bgpvpn.route_targets,
                'tenant_id': bgpvpn.tenant_id}

        ret_dict = {'bgpvpn': data}
        self.mock_create_bgpvpn.return_value = ret_dict

        ret_val = api.bgpvpn.bgpvpn_create(self.request, **data)

        self.mock_create_bgpvpn.assert_called_once_with(body=ret_dict)

        self.assertIsInstance(ret_val, api.bgpvpn.Bgpvpn)
        self.assertEqual(bgpvpn.name, ret_val.name)

    @test.create_mocks({neutronclient: ('show_bgpvpn',)})
    def test_bgpvpn_get(self):
        bgpvpn = self.bgpvpns.first()
        ret_dict = {'bgpvpn': self.api_bgpvpns.first()}

        self.mock_show_bgpvpn.return_value = ret_dict

        ret_val = api.bgpvpn.bgpvpn_get(self.request, bgpvpn.id)
        self.assertIsInstance(ret_val, api.bgpvpn.Bgpvpn)
        self.assertEqual(bgpvpn.name, ret_val.name)

    @test.create_mocks({neutronclient: ('update_bgpvpn',)})
    def test_bgpvpn_update(self):
        bgpvpn = self.bgpvpns.first()
        bgpvpn_dict = self.api_bgpvpns.first()

        bgpvpn.name = 'new name'
        bgpvpn.route_targets = ['65001:2']

        bgpvpn_dict['name'] = 'new name'
        bgpvpn_dict['route_targets'] = ['65001:2']

        data = {'name': bgpvpn.name,
                'route_targets': bgpvpn.route_targets}
        ret_dict = {'bgpvpn': bgpvpn_dict}

        self.mock_update_bgpvpn.return_value = ret_dict

        ret_val = api.bgpvpn.bgpvpn_update(self.request, bgpvpn.id, **data)
        self.assertIsInstance(ret_val, api.bgpvpn.Bgpvpn)
        self.assertEqual('new name', ret_val.name)
        self.assertEqual(['65001:2'], ret_val.route_targets)

    @test.create_mocks({neutronclient: ('delete_bgpvpn',)})
    def test_bgpvpn_delete(self):
        bgpvpn = self.bgpvpns.first()
        api.bgpvpn.bgpvpn_delete(self.request, bgpvpn.id)

        self.mock_delete_bgpvpn.assert_called_once_with(bgpvpn.id)

    @test.create_mocks({
        neutronclient: ('show_bgpvpn_network_assoc',)})
    def test_network_association_get(self):
        bgpvpn = self.bgpvpns.first()
        na = self.network_associations.first()
        ret_dict = {
            'network_association': self.api_network_associations.first()}

        self.mock_show_bgpvpn_network_assoc.return_value = ret_dict

        ret_val = api.bgpvpn.network_association_get(
            self.request, bgpvpn.id, na.id)
        self.assertIsInstance(ret_val, api.bgpvpn.NetworkAssociation)

    @test.create_mocks({
        neutronclient: ('list_bgpvpn_network_assocs',)})
    def test_network_association_list(self):
        exp_nas = self.network_associations.list()

        api_na = {'network_associations': self.api_network_associations.list()}
        self.mock_list_bgpvpn_network_assocs.return_value = api_na

        ret_vals = api.bgpvpn.network_association_list(self.request, 'dummy')
        for (ret_val, exp_na) in zip(ret_vals, exp_nas):
            self.assertIsInstance(ret_val, api.bgpvpn.NetworkAssociation)
            self.assertEqual(exp_na.id, ret_val.id)
            self.assertEqual(exp_na.network_id, ret_val.network_id)

    @test.create_mocks({
        neutronclient: ('create_bgpvpn_network_assoc',)})
    def test_network_association_create(self):
        bgpvpn = self.bgpvpns.first()
        network = self.networks.first()
        data = {'network_id': network.id}
        ret_dict = {'network_association': data}

        self.mock_create_bgpvpn_network_assoc.return_value = ret_dict

        ret_val = api.bgpvpn.network_association_create(
            self.request, bgpvpn.id, **data)

        self.assertIsInstance(ret_val, api.bgpvpn.NetworkAssociation)
        self.assertEqual(network.id, ret_val.network_id)

    @test.create_mocks({
        neutronclient: ('delete_bgpvpn_network_assoc',)})
    def test_network_association_delete(self):
        bgpvpn = self.bgpvpns.first()
        na = self.network_associations.first()

        api.bgpvpn.network_association_delete(self.request, na.id, bgpvpn.id)

        self.mock_delete_bgpvpn_network_assoc.assert_called_once_with(
            bgpvpn.id, na.id)

    @test.create_mocks({
        neutronclient: ('show_bgpvpn_router_assoc',)})
    def test_router_association_get(self):
        bgpvpn = self.bgpvpns.first()
        na = self.router_associations.first()
        ret_dict = {
            'router_association': self.api_router_associations.first()}

        self.mock_show_bgpvpn_router_assoc.return_value = ret_dict

        ret_val = api.bgpvpn.router_association_get(
            self.request, bgpvpn.id, na.id)
        self.assertIsInstance(ret_val, api.bgpvpn.RouterAssociation)

    @test.create_mocks({
        neutronclient: ('list_bgpvpn_router_assocs',)})
    def test_router_association_list(self):
        exp_nas = self.router_associations.list()

        api_na = {'router_associations': self.api_router_associations.list()}
        self.mock_list_bgpvpn_router_assocs.return_value = api_na

        ret_vals = api.bgpvpn.router_association_list(self.request, 'dummy')
        for (ret_val, exp_na) in zip(ret_vals, exp_nas):
            self.assertIsInstance(ret_val, api.bgpvpn.RouterAssociation)
            self.assertEqual(exp_na.id, ret_val.id)
            self.assertEqual(exp_na.router_id, ret_val.router_id)

    @test.create_mocks({
        neutronclient: ('create_bgpvpn_router_assoc',)})
    def test_router_association_create(self):
        bgpvpn = self.bgpvpns.first()
        router = self.routers.first()
        data = {'router_id': router.id}
        ret_dict = {'router_association': data}

        self.mock_create_bgpvpn_router_assoc.return_value = ret_dict

        ret_val = api.bgpvpn.router_association_create(
            self.request, bgpvpn.id, **data)

        self.assertIsInstance(ret_val, api.bgpvpn.RouterAssociation)
        self.assertEqual(router.id, ret_val.router_id)

    @test.create_mocks({
        neutronclient: ('delete_bgpvpn_router_assoc',)})
    def test_router_association_delete(self):
        bgpvpn = self.bgpvpns.first()
        na = self.router_associations.first()

        api.bgpvpn.router_association_delete(self.request, na.id, bgpvpn.id)

        self.mock_delete_bgpvpn_router_assoc.assert_called_once_with(
            bgpvpn.id, na.id)



# Copyright (c) 2017 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms


from bgpvpn_dashboard.api import bgpvpn as bgpvpn_api

LOG = logging.getLogger(__name__)


class UpdateRouterAssociation(forms.SelfHandlingForm):
    bgpvpn_id = forms.CharField(widget=forms.HiddenInput())
    router_association_id = forms.CharField(
        label=_("ID"), widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    router_id = forms.CharField(
        label=_("Router"),
        widget=forms.TextInput(attrs={'readonly': 'readonly'}))
    advertise_extra_routes = forms.BooleanField(
        label=_("Advertise Extra Routes"),
        initial=True,
        required=False,
        help_text="Boolean flag controlling whether or not the routes "
                  "specified in the routes attribute of the router will be "
                  "advertised to the BGPVPN (default: true).")

    def __init__(self, request, *args, **kwargs):
        super(UpdateRouterAssociation, self).__init__(request, *args, **kwargs)
        self.action = 'update'

    def handle(self, request, data):
        router_association_id = data['router_association_id']
        bgpvpn_id = data['bgpvpn_id']
        try:
            bgpvpn_api.router_association_update(
                request, bgpvpn_id, router_association_id,
                advertise_extra_routes=data['advertise_extra_routes'])
            return True
        except exceptions as e:
            exceptions.handle(
                request,
                "Unable to update bgpvpn router association %s:" % str(e))
            return False



# Copyright (c) 2017 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import bgpvpn_dashboard.api.bgpvpn as bgpvpn_api

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tabs

from bgpvpn_dashboard.dashboards.project.bgpvpn.network_associations \
    import tabs as network_tabs
from bgpvpn_dashboard.dashboards.project.bgpvpn.router_associations \
    import tabs as router_tabs


class OverviewTab(tabs.Tab):
    name = _("Overview")
    slug = "overview"
    template_name = ("project/bgpvpn/_detail_overview.html")
    preload = False

    def _get_data(self):
        bgpvpn = {}
        bgpvpn_id = None
        try:
            bgpvpn_id = self.tab_group.kwargs['bgpvpn_id']
            bgpvpn = bgpvpn_api.bgpvpn_get(self.request, bgpvpn_id)
            bgpvpn.set_id_as_name_if_empty(length=0)
        except Exception:
            msg = _('Unable to retrieve details for bgpvpn "%s".') % (
                bgpvpn_id)
            exceptions.handle(self.request, msg)
        return bgpvpn

    def get_context_data(self, request, **kwargs):
        context = super(OverviewTab, self).get_context_data(request, **kwargs)
        bgpvpn = self._get_data()
        context["bgpvpn"] = bgpvpn
        return context


class BgpvpnDetailsTabs(tabs.DetailTabsGroup):
    slug = "bgpvpn_tabs"
    tabs = (OverviewTab,
            network_tabs.NetworkAssociationsTab,
            router_tabs.RouterAssociationsTab)
    sticky = True
